@@ -256,11 +256,16 @@ class Ecobee(object):
         self,
         index: int,
         fan_mode: str,
-        cool_temp: int,
-        heat_temp: int,
-        hold_type: str = "nextTransition",
+        hold_type: str,
+        **optional_arg,
     ) -> None:
-        """Sets the fan mode (auto, minontime, on)."""
+        """
+        Sets the fan mode (auto, minontime, on).
+            valid optional_arg
+                holdHours - required if HoldType is holdHours
+                coolHoldTemp
+                heatHoldTemp
+        """
         body = {
             "selection": {
                 "selectionType": "thermostats",
@@ -271,13 +276,22 @@ class Ecobee(object):
                     "type": "setHold",
                     "params": {
                         "holdType": hold_type,
-                        "coolHoldTemp": int(cool_temp * 10),
-                        "heatHoldTemp": int(heat_temp * 10),
                         "fan": fan_mode,
                     },
                 }
             ],
         }
+
+        # Set the optional args
+        if "holdHours" in optional_arg:
+            # Required if and only if hold_type == holdHours
+            if hold_type == "holdHours":
+                body["functions"][0]["params"]["holdHours"] = int(optional_arg["holdHours"])
+        if "coolHoldTemp" in optional_arg:
+            body["functions"][0]["params"]["coolHoldTemp"] = int(optional_arg["coolHoldTemp"]) * 10
+        if "heatHoldTemp" in optional_arg:
+            body["functions"][0]["params"]["heatHoldTemp"] = int(optional_arg["heatHoldTemp"]) * 10
+
         log_msg_action = "set fan mode"
 
         try:
@@ -337,7 +351,7 @@ class Ecobee(object):
             raise err
 
     def set_climate_hold(
-        self, index: int, climate: str, hold_type: str = "nextTransition"
+        self, index: int, climate: str, hold_type: str = "nextTransition", hold_hours: int = None
     ) -> None:
         """Sets a climate hold (away, home, sleep)."""
         body = {
@@ -348,10 +362,14 @@ class Ecobee(object):
             "functions": [
                 {
                     "type": "setHold",
-                    "params": {"holdType": hold_type, "holdClimateRef": climate},
+                    "params": {"holdType": hold_type, "holdClimateRef": climate, "holdHours": hold_hours},
                 }
             ],
         }
+
+        if hold_type != "holdHours":
+            del body["functions"][0]["params"]["holdHours"]
+
         log_msg_action = "set climate hold"
 
         try:
@@ -389,8 +407,8 @@ class Ecobee(object):
                         "startTime": start_time,
                         "endDate": end_date,
                         "endTime": end_time,
-                        "fan_mode": fan_mode,
-                        "fan_min_on_time": fan_min_on_time,
+                        "fan": fan_mode,
+                        "fanMinOnTime": fan_min_on_time,
                     },
                 }
             ],
@@ -455,6 +473,22 @@ class Ecobee(object):
         except (ExpiredTokenError, InvalidTokenError) as err:
             raise err
 
+    def set_humidifier_mode(self, index: int, humidifier_mode: str) -> None:
+        """Sets the humidifier mode (auto, off, manual)."""
+        body = {
+            "selection": {
+                "selectionType": "thermostats",
+                "selectionMatch": self.thermostats[index]["identifier"],
+            },
+            "thermostat": {"settings": {"humidifierMode": humidifier_mode}},
+        }
+        log_msg_action = "set humidifier mode"
+
+        try:
+            self._request("POST", ECOBEE_ENDPOINT_THERMOSTAT, log_msg_action, body=body)
+        except (ExpiredTokenError, InvalidTokenError) as err:
+            raise err
+
     def set_humidity(self, index: int, humidity: str) -> None:
         """Sets target humidity level."""
         body = {
@@ -608,9 +642,14 @@ class Ecobee(object):
                 f"Connection to ecobee timed out while attempting to {log_msg_action}. "
                 f"Possible connectivity outage."
             )
-        except (RequestException, json.decoder.JSONDecodeError):
+        except json.decoder.JSONDecodeError:
+            _LOGGER.error(
+                f"Error decoding response from ecobee while attempting to {log_msg_action}. "
+            )
+        except RequestException as err:
             _LOGGER.error(
                 f"Error connecting to ecobee while attempting to {log_msg_action}. "
-                f"Possible connectivity outage."
+                f"Possible connectivity outage.\n"
+                f"{err}"
             )
         return None
